source("plotProzess.r")
#------------------
# Input-Parameter:
#
# p, q, r      Wahrscheinlichkeiten, 
#              -> also Werte zwischen 0 und 1 mit p+q+r=1
# n            Pfadlaenge
# plot         Plot-Parameter fuer Visualisierung des Pfades in
#              einer Treppenfunktion -> TRUE/FALSE
# ...          Weitere Parameter fuer plot Funktion

#------------------
# Output:
# data.frame mit zwei Spalten:
#             1. Spalte: Zeitpunkte
#             2. Spalte: Pfad

einfacheIrrfahrt <- function(n = 20, p = 0.4, q = 0.4, r = 0.2,
                             plot = TRUE, ...) {
  
  # Konsitenzcheck
  if(any(c(p, q, r) > 1) || any(c(p, q, r) < 0) || (sum(p, q, r) != 1)) {
    stop("p, q, r m\u00FCssen Wahrscheinlichkeiten sein mit Summe 1!\n")
  }
  
  # Pfad der Irrfahrt erzeugen, Startpunkt in 0
  pfad <- numeric(n + 1)
  for(i in 1:n){
    pfad[i + 1] <- pfad[i] + sample(c(-1, 0, 1), 
                                    size = 1, prob = c(q, r, p))
  }
  
  # Alternative ueber Zuwaechse:
  # pfad <- c(0,cumsum(sample(c(-1,0,1), size = n, 
  #                           prob = c(q,r,p), replace =TRUE)))
  
  # Zusammenfuegen von Zeitpunkten und Pfad in einer Datenmatrix
  irrfahrt <- data.frame(zeit = (0:n), pfad = pfad)
  
  # Visualisieren des Pfades, falls Parameter plot==TRUE
  if(plot){
    # Plotten des Prozesses 
    plotProzess(irrfahrt$zeit, irrfahrt$pfad, xlab = "t",
                ylab = expression(X[t]),
                ylim = c(- max(p, q) * n, max(p, q) * n), 
                main = paste("p =", eval(expression(p)), 
                             "   q =", eval(expression(q)), 
                             "   r =", eval(expression(r))), ...)
	  return(invisible(irrfahrt))
  }
  
  return(irrfahrt)
}

#==================
# Veraenderung der Funktion zur Irrfahrt mit absorbierenden Schranken

#------------------
# Input-Parameter:
# p, q, r      Wahrscheinlichkeiten, 
#              -> also Werte zwischen 0 und 1 mit p+q+r=1
# a, b         Schranken fuer obere und untere Grenze
# n            maximale Pfadlaenge
#              -> Fuer den Fall, dass wir irgendwann abbrechen wollen
# plot         Plot-Parameter fuer Visualisierung des Pfades in
#              einer Treppenfunktion 
#              -> TRUE/FALSE
# ...          Weitere Parameter fuer plot Funktion


#------------------
# Output:
# data.frame mit zwei Spalten:
#             1. Spalte: Zeitpunkte
#             2. Spalte: Pfad

schrankenIrrfahrt <- function(a = -5, b = 5, n = 20, p = 0.5, q = 0.5, r = 0, plot = TRUE, ...){
  
  # Konsitenzcheck
  if(any(c(p, q, r) > 1) || any(c(p, q, r) < 0) || (sum(p, q, r) != 1)) {
    stop("p, q, r m\u00FCssen Wahrscheinlichkeiten sein mit Summe 1!\n")
  }
  if(a >= b) {
    stop("a muss kleiner als b sein!\n")
  }
  
  # Pfad intialisieren mit Startwert 0
  pfad <- numeric(n + 1)
  
  # Laufparameter fuer die while-Schleife initialisieren
  i <- 1
  
  # Solange der Pfad nicht bei a oder b angekommen ist 
  # und die maximale Weite erzielt ist, geht die Irrfahrt weiter
  while((pfad[i] > a) && (pfad[i] < b) && (i < (n + 1))){
    pfad[i + 1] <- pfad[i] + sample(c(-1, 0, 1), size = 1, prob = c(q, r, p))
    i <- i + 1
  }         
  
  # Pfad am Ende abschneiden
  if((i + 1) <= (n + 1)) {
    pfad <- pfad[-((i + 1):(n + 1))]
  }
  # Neue Pfadlaenge bestimmen
  n <- length(pfad)
  
  # data.frame fuer Prozess erzeugen
  irrfahrt <- data.frame(zeit = 0:(n-1), pfad = pfad)
  
  if(plot){
  	# Plotten des Prozesses 
	  plotProzess(irrfahrt$zeit, irrfahrt$pfad, xlab = "t",
                ylab = expression(X[t]), ylim = c(a,b), 
                main = paste("p =", eval(expression(p)), 
                             "   q =", eval(expression(q)), 
                             "   r =", eval(expression(r))), ...)
	  return(invisible(irrfahrt))
  }
  
  return(irrfahrt)
  
}

#-----------------
# Input-Parameter:
# pfad       Pfad einer einfachen Irrfahrt auf der Geraden

#----------------
# Output:
# Vektor mit ML-Schaetzern p.ML, q.ML sowie r.ML

MLirrfahrt <- function(pfad){
  
  # Anzahl der Uebergaenge der Irrfahrt bestimmen
  nges <- length(pfad) - 1
  
  # Auszaehlung der Differenzen zwischen 2 Zeitpunkten
  tdiff <- table(diff(pfad))
  names(tdiff) <- c("qML", "rML", "pML")
  
  # Relative Haeufigkeiten als ML-Schaetzer zurueckgeben
  return(pqr / nges)
  
}

#------------------
# Input-Parameter:
#
# p            Wahrscheinlichkeit fuer positiven Zuwachs 
# tmax         Zeitpunkt, bis zu dem der Prozess simuliert werden soll
# sigma        Varianzparameter
# const        Konstante c
# plot         Plot-Parameter fuer Visualisierung des Pfades 
#              -> TRUE/FALSE
# ...          Weitere Parameter fuer plot Funktion

#-----------------
# Output:
# data.frame mit zwei Spalten:
#             1. Spalte: Zeitpunkte
#             2. Spalte: Pfad

varIrrfahrt <- function(p = 0.4, sigma = 1, const = 100, plot = TRUE, tmax = 10, ...){
  
  # Konsitenzcheck
  if((p > 1) || (p < 0)) {
    stop("p muss Wahrscheinlichkeit sein!\n")
  }
  
  # deltax und deltat berechenen
  deltax <- sigma / sqrt(const)
  deltat <- 1 / const
  n <- tmax / deltat
  
  # Zeitvektor initialisieren als Folge von deltat
  zeit <- seq(0, n * deltat, length = n + 1)
  
  # Pfad intialisieren
  pfad <- numeric(n + 1)

  # Pfad erzeugen wie gehabt, nur mit (+-deltax) 
  # als Schrittweiten anstelle von 1
  pfad <- cumsum(c(0, sample(c(-deltax, deltax), size = n,
                             prob = c(1 - p, p), replace = TRUE)))
  
  # Zusammenfuegen von Zeitpunkten und Pfad in einer Datenmatrix
  irrfahrt <- data.frame(zeit = zeit, pfad = pfad)
  
  # Visualisieren des Pfades, falls Parameter plot==TRUE
  if(plot){
    # Plotten des Prozesses 
    plotProzess(irrfahrt$zeit, irrfahrt$pfad, xlab = "t",
                ylab = expression(X[t]),
                main = paste("c =", eval(expression(const)), "   ",
                             expression(sigma), "=", 
                             eval(expression(sigma))), ...)
    return(invisible(irrfahrt))
  }
  
  return(irrfahrt)
}
